var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var Heap = /** @class */ (function () {
    function Heap(compare) {
        this.arr = [0];
        this.defaultCompare = function (a, b) { return a < b; };
        this.compare = (typeof compare === 'function') ? compare : this.defaultCompare;
    }
    Object.defineProperty(Heap.prototype, "size", {
        /**
        * 堆中元素数量
        */
        get: function () {
            return this.arr.length - 1;
        },
        enumerable: false,
        configurable: true
    });
    /**
     * 根据可迭代对象生成堆
     * @param {*} data iterable 对象
     * @param {*} compare
     */
    Heap.heapify = function (data, compare) {
        var e_1, _a;
        var heap = new Heap(compare);
        try {
            for (var data_1 = __values(data), data_1_1 = data_1.next(); !data_1_1.done; data_1_1 = data_1.next()) {
                var item = data_1_1.value;
                heap.push(item);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (data_1_1 && !data_1_1.done && (_a = data_1.return)) _a.call(data_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return heap;
    };
    /**
    * 入队
    */
    Heap.prototype.push = function (item) {
        var arr = this.arr;
        arr.push(item); //添加到末尾
        this.up(arr.length - 1); // 尝试上浮
    };
    /**
    * 向上调整
    */
    Heap.prototype.up = function (k) {
        var _a = this, arr = _a.arr, compare = _a.compare, parent = _a.parent;
        while (k > 1 && compare(arr[k], arr[parent(k)]) > 0) {
            this.swap(parent(k), k);
            k = parent(k);
        }
    };
    /**
    * 交换位置
    */
    Heap.prototype.swap = function (i, j) {
        var _a;
        var arr = this.arr;
        _a = __read([arr[j], arr[i]], 2), arr[i] = _a[0], arr[j] = _a[1];
    };
    /**
    * 左孩子
    */
    Heap.prototype.left = function (k) { return k * 2; };
    /**
    * 右孩子
    */
    Heap.prototype.right = function (k) { return k * 2 + 1; };
    /**
    * 双亲
    */
    Heap.prototype.parent = function (k) { return Math.floor(k / 2); };
    /**
     * 向下调整
     * @param {int} k
     */
    Heap.prototype.down = function (k) {
        var _a = this, arr = _a.arr, compare = _a.compare, left = _a.left, right = _a.right;
        var size = this.size;
        // 如果沉到堆底，就沉不下去了
        while (left(k) <= size) {
            var child = left(k);
            if (right(k) <= size && compare(arr[right(k)], arr[child]) > 0) {
                child = right(k); // 选择左右子节点中更靠近堆顶的，这样能维持下沉后原本的 left与right 之间的顺序关系
            }
            // 如果当前的k比子节点更靠近堆顶，不用下沉了
            if (compare(arr[k], arr[child]) > 0)
                return;
            // 下沉
            this.swap(k, child);
            k = child;
        }
    };
    /**
    * 返回优先级最高元素，出队
    */
    Heap.prototype.pop = function () {
        if (this.size === 0)
            return null; //行为同Java的PriorityQueue
        var arr = this.arr;
        this.swap(1, arr.length - 1); // 末尾的换上来，堆顶放到最后等待返回
        var res = arr.pop();
        this.down(1); // 换上来的末尾尝试下沉
        return res;
    };
    /**
    * 返回优先级最高元素，但不出队
    */
    Heap.prototype.peek = function () {
        return this.arr[1];
    };
    return Heap;
}());
export { Heap };
//# sourceMappingURL=heap.js.map